/* ============================================================
 *
 * This file is a part of educazionik project
 * http://educazionik.sourceforge.net/
 *
 * Date        : 1/1/2003
 * Description : Implementation class anafrase
 *
 * Copyright 2003-2009 by Nicola De Filippo <nicola@nicoladefilippo.it>
 * Copyright 2009      by Angelo Naselli    <anaselli at linux dot it>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * ============================================================ */
#include "anafrase.h"

#include <qvariant.h>
#include <qlabel.h>
#include <qlcdnumber.h>
#include <qpushbutton.h>
#include <qmime.h>
#include <QDrag> // qdragobject.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qapplication.h>
#include <qmessagebox.h>

#include "aboutdlg.h"
#include "s_anafrase.h"
#include "anafrase_version.h"


/*
 *  Constructs a anafrase which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
anafrase::anafrase( QWidget* parent)
        : QDialog( parent), Ui_anafrase()
{
  setupUi(this);
  p=NULL;
  m_title = QString(APPL_NAME)    + " V. " +
            QString(APPL_VERSION);
  setWindowIcon(QIcon( QPixmap(QString(RESOURCE_DIR) + "/educazionik.png")));
  setWindowTitle(m_title);

  // signals and slots connections
  connect( PushButton1, SIGNAL( pressed() ), this, SLOT( vero() ) );
  connect( PushButton2, SIGNAL( pressed() ), this, SLOT( falso() ) );
  connect( PushButton3, SIGNAL( pressed() ), this, SLOT( probabile() ) );
  connect( m_info, SIGNAL( pressed() ), this, SLOT( info() ) );
  connect( m_resetButton, SIGNAL( pressed() ), this, SLOT( azzera() ) );
  connect( m_exit, SIGNAL( pressed() ), this, SLOT( esci() ) );
  connect( PushButton7, SIGNAL( pressed() ), this, SLOT( senzanegazione() ) );
  connect( PushButton8, SIGNAL( pressed() ), this, SLOT( conegazione() ) );

  QPalette palette;
  palette.setColor(TextLabel1->backgroundRole(), QColor( 255, 255, 255 ));
  TextLabel1->setPalette(palette);

  TextLabel2->setGeometry( QRect( 10, 160, 241, 80 ) );
  palette.setColor(TextLabel2->backgroundRole(), QColor( 255, 255, 255 ));
  TextLabel2->setPalette(palette);

  palette.setColor(LCD_Wrong->foregroundRole(), QColor( 170, 0, 0 ));
  LCD_Wrong->setPalette(palette);
  LCD_Wrong->setSegmentStyle( QLCDNumber::Filled );

  palette.setColor(LCD_Correct->foregroundRole(), QColor( 0, 170, 0 ));
  LCD_Correct->setPalette(palette);
  LCD_Correct->setSegmentStyle( QLCDNumber::Filled );

  palette.setColor(RightLabel->foregroundRole(), QColor( 0, 85, 0 ));
  RightLabel->setPalette(palette);

  palette.setColor(WrongLabel->foregroundRole(), QColor( 170, 0, 0 ));
  WrongLabel->setPalette(palette);

  QString filename = QString(DATA_DIR) + "/frase.txt";
  init(filename);

}

void anafrase::init(QString path)
{
    QString f,ip;
    if (!p)
      p = new s_anafrase(path);

    if (p->errore() !=-1)
    {
        p->frase(f);
        p->ipotesi(ip);
        TextLabel1->setText(f);
        TextLabel2->setText(ip);
        LCD_Correct->display(0);
        LCD_Wrong->display(0);
        mResultIcon->clear();

    }
    else
    {
        QMessageBox::critical( 0, m_title, QString("Non riesco a trovare il file ")+path);

//         PushButton1->hide();
//         PushButton2->hide();
//         PushButton3->hide();
//         PushButton4->hide();
//         PushButton5->hide();
//         PushButton6->hide();

        LCD_Correct->hide();
        LCD_Wrong->hide();
//         TextLabel2->hide();
//         TextLabel1->hide();
        delete p;
        p=NULL;

    }
}



/*
 *  Destroys the object and frees any allocated resources
 */
anafrase::~anafrase()
{
  delete p;
}

void anafrase::azzera()
{
    LCD_Correct->display(0);
    LCD_Wrong->display(0);
    mResultIcon->clear();
}

void anafrase::conegazione()
{
    delete p;
    p = NULL;
    init(QString(DATA_DIR) + "/frasenon.txt");
}

void anafrase::esci()
{
    QApplication::exit();
}

void anafrase::falso()
{
    QString ip,f;
    if (p->esatto()==0)
    {
        //incrementa lcd
        LCD_Correct->display(LCD_Correct->intValue()+1);
        mResultIcon->setPixmap(QPixmap(QString(RESOURCE_DIR) + "/smile.png"));
        if (p->ipotesi(ip)!=-1)
            TextLabel2->setText(ip);
        else
        {
            /*nuova frase
            nuova ipotesi*/
            p->frase(f);
            p->ipotesi(ip);
            TextLabel1->setText(f);
            TextLabel2->setText(ip);
        }
    }
    else
    {
        // incrementa lcd -
        LCD_Wrong->display(LCD_Wrong->intValue()+1);
        mResultIcon->setPixmap(QPixmap(QString(RESOURCE_DIR) + "/frown.png"));
    }
}

void anafrase::info()
{
  QString info = "(c) 2003 <b>Nicola De Filippo</b> - nicola@nicoladefilippo.it<br>"
                 "(c) 2009 <b>Angelo Naselli</b>    - anaselli@linux.it<br>";
  info += "http://educazionik.sourceforge.net";
  QString description = "Programma di esercizi di comprensione della frase";
  QString icon = QString(RESOURCE_DIR) + "/logo.png";

  //AboutDialog about(this, m_title, info, description, icon);
  AboutDialog about(this, m_title, "", description, icon, "", "", "");
  about.exec();
}

void anafrase::senzanegazione()
{
    delete p;
    p = NULL;
    init(QString(DATA_DIR) + "/frase.txt");
}

void anafrase::probabile()
{
    QString ip,f;
    if (p->esatto()==2)
    {
        //incrementa lcd
        LCD_Correct->display(LCD_Correct->intValue()+1);
        mResultIcon->setPixmap(QPixmap(QString(RESOURCE_DIR) + "/smile.png"));
        if (p->ipotesi(ip)!=-1)
            TextLabel2->setText(ip);
        else
        {
            p->frase(f);
            p->ipotesi(ip);
            TextLabel1->setText(f);
            TextLabel2->setText(ip);
        }
    }
    else
    {
        // incrementa lcd-
        LCD_Wrong->display(LCD_Wrong->intValue()+1);
        mResultIcon->setPixmap(QPixmap(QString(RESOURCE_DIR) + "/frown.png"));
    }
}


void anafrase::vero()
{
    QString ip,f;
    if (p->esatto()==1)
    {
        //incrementa lcd
        LCD_Correct->display(LCD_Correct->intValue()+1);
        mResultIcon->setPixmap(QPixmap(QString(RESOURCE_DIR) + "/smile.png"));
        if (p->ipotesi(ip)!=-1)
            TextLabel2->setText(ip);
        else
        {
            //nuova frase
            //nuova ipotesi
            p->frase(f);
            p->ipotesi(ip);
            TextLabel1->setText(f);
            TextLabel2->setText(ip);

        }
    }
    else
    {
        LCD_Wrong->display(LCD_Wrong->intValue()+1);
        // incrementa lcd-
        mResultIcon->setPixmap(QPixmap(QString(RESOURCE_DIR) + "/frown.png"));
    }
}



