/* ============================================================
 *
 * This file is a part of educazionik project
 * http://educazionik.sourceforge.net/
 *
 * Date        : 13-8-2002
 * Description : Lessico implementation
 *
 * Copyright 2002-2009 by Nicola De Filippo <nicola@nicoladefilippo.it>
 * Copyright 2009      by Angelo Naselli    <anaselli at linux dot it>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#include<qmessagebox.h>
#include<time.h>
#include "form10.h"
#include "ui_form9.h"
#include "terna.h"
#include "lessico_version.h"

struct privateForm10
{
    terna *scelta; 
    QString filename;
    QString title;
    
    privateForm10() : scelta(NULL) {}
    ~privateForm10()
    {
      if (scelta)
        delete scelta;
    }
};

Form10::Form10(QWidget *parent)
        : QDialog(parent), ui(new Ui::Form9), d(new privateForm10)
{
  ui->setupUi(this);
  d->title = QString(APPL_NAME)    + " V. " +
             QString(APPL_VERSION) + " "    +
             QString(APPL_DATE)    + " "    +
             QString(APPL_TIME);
  d->filename = QString(DATA_DIR) + "/aree2.txt";

  setWindowTitle("Parola intrusa");
  ui->m_aim_label->setText("Clicca sulla parola intrusa");

  connect(ui->m_resultTable->m_exit, SIGNAL(pressed()), this, SLOT(esci()));
  connect(ui->m_resultTable->m_go_on, SIGNAL(pressed()), this, SLOT(avanti()));
  connect(ui->m_resultTable->m_resetButton, SIGNAL(pressed()), this, SLOT(azzera()));
}

Form10::~Form10()
{
    delete ui;
    delete d;
}

void Form10::init()
{
  if (!d->scelta)
    d->scelta=new terna(d->filename);

  if (d->scelta->errore()!=-1)
  {
      azzera();
  }
  else
  {
    QMessageBox::critical( 0, d->title, QString("Non riesco a trovare il file ") + d->filename);
  }
}

void Form10::azzera()
{
  avanti();
    
  ui->m_resultTable->LCD_Wrong->display(0);
  ui->m_resultTable->LCD_Correct->display(0);
}

void Form10::avanti()
{
    QString a[4];
    d->scelta->frase(&a[0]);
    ui->pushButton4->setText(a[0]);
    ui->pushButton5->setText(a[1]);
    ui->pushButton6->setText(a[2]);
    ui->pushButton7->setText(a[3]);
    QPalette palette;
    palette.setColor(ui->pushButton4->backgroundRole(), QColor( 255, 255, 255 ));
    ui->pushButton4->setPalette(palette);
    palette.setColor(ui->pushButton5->backgroundRole(), QColor( 255, 255, 255 ));
    ui->pushButton5->setPalette(palette);
    palette.setColor(ui->pushButton6->backgroundRole(), QColor( 255, 255, 255 ));
    ui->pushButton6->setPalette(palette);
    palette.setColor(ui->pushButton7->backgroundRole(), QColor( 255, 255, 255 ));
    ui->pushButton7->setPalette(palette);

    ui->pushButton4->setEnabled(true);
    ui->pushButton5->setEnabled(true);
    ui->pushButton6->setEnabled(true);
    ui->pushButton7->setEnabled(true);
    ui->m_resultTable->m_go_on->setEnabled(false);
}

void Form10::esci()
{
  ui->m_resultTable->m_go_on->setEnabled(false);

  this->accept();
}

void Form10::primo()
{
    if (d->scelta->esatto(ui->pushButton4->text())>-1)
    {
        ui->m_resultTable->LCD_Correct->display(ui->m_resultTable->LCD_Correct->intValue()+1);
        ui->m_resultTable->m_go_on->setEnabled(true);
        ui->pushButton4->setEnabled(false);
        ui->pushButton5->setEnabled(false);
        ui->pushButton6->setEnabled(false);
        ui->pushButton7->setEnabled(false);
        QPalette palette;
        palette.setColor(ui->pushButton4->backgroundRole(), QColor( 255, 255, 127 ) );
        ui->pushButton4->setPalette(palette);
    }
    else
        ui->m_resultTable->LCD_Wrong->display(ui->m_resultTable->LCD_Wrong->intValue()+1);

}

void Form10::secondo()
{
    if (d->scelta->esatto(ui->pushButton5->text())>-1)
    {
        ui->m_resultTable->LCD_Correct->display(ui->m_resultTable->LCD_Correct->intValue()+1);
        ui->m_resultTable->m_go_on->setEnabled(true);
        ui->pushButton4->setEnabled(false);
        ui->pushButton5->setEnabled(false);
        ui->pushButton6->setEnabled(false);
        ui->pushButton7->setEnabled(false);
        QPalette palette;
        palette.setColor(ui->pushButton5->backgroundRole(), QColor( 255, 255, 127 ) );
        ui->pushButton5->setPalette(palette);
    }
    else
        ui->m_resultTable->LCD_Wrong->display(ui->m_resultTable->LCD_Wrong->intValue()+1);
}

void Form10::terzo()
{
    if (d->scelta->esatto(ui->pushButton6->text())>-1)
    {
        ui->m_resultTable->LCD_Correct->display(ui->m_resultTable->LCD_Correct->intValue()+1);
        ui->m_resultTable->m_go_on->setEnabled(true);
        ui->pushButton4->setEnabled(false);
        ui->pushButton5->setEnabled(false);
        ui->pushButton6->setEnabled(false);
        ui->pushButton7->setEnabled(false);
        QPalette palette;
        palette.setColor(ui->pushButton6->backgroundRole(), QColor( 255, 255, 127 ) );
        ui->pushButton6->setPalette(palette);
    }
    else
        ui->m_resultTable->LCD_Wrong->display(ui->m_resultTable->LCD_Wrong->intValue()+1);
}

void Form10::quarto()
{
    if (d->scelta->esatto(ui->pushButton7->text())>-1)
    {
        ui->m_resultTable->LCD_Correct->display(ui->m_resultTable->LCD_Correct->intValue()+1);
        ui->m_resultTable->m_go_on->setEnabled(true);
        ui->pushButton4->setEnabled(false);
        ui->pushButton5->setEnabled(false);
        ui->pushButton6->setEnabled(false);
        ui->pushButton7->setEnabled(false);
        QPalette palette;
        palette.setColor(ui->pushButton7->backgroundRole(), QColor( 255, 255, 127 ) );
        ui->pushButton7->setPalette(palette);
    }
    else
        ui->m_resultTable->LCD_Wrong->display(ui->m_resultTable->LCD_Wrong->intValue()+1);
}
