/* ============================================================
 *
 * This file is a part of educazionik project
 * http://educazionik.sourceforge.net/
 *
 * Date        : 13-8-2002
 * Description : Lessico implementation
 *
 * Copyright 2002-2009 by Nicola De Filippo <nicola@nicoladefilippo.it>
 * Copyright 2009      by Angelo Naselli    <anaselli at linux dot it>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#include <QMessageBox>
#include <time.h>
#include "form7.h"
#include "ui_form7.h"
#include "forlessico.h"
#include "lessico_version.h"

struct privateForm7
{
    forlessico *scelta;
    int indovinati;
    QString title;
    QString filename;
    
    privateForm7() : scelta(NULL) {}
    ~privateForm7()
    {
      if (scelta)
        delete scelta;
    }
};

Form7::Form7(QWidget *parent)
        : QDialog(parent), ui(new Ui::Form7), d(new privateForm7)
{
  ui->setupUi(this);
  
  d->title = QString(APPL_NAME)    + " V. " +
              QString(APPL_VERSION) + " "    +
              QString(APPL_DATE)    + " "    +
              QString(APPL_TIME);
  d->filename = QString(DATA_DIR) + "/lessico.txt";
  
  setWindowTitle("Lessico");
  
  connect( ui->m_resultTable->m_resetButton, SIGNAL( pressed() ), this, SLOT( azzera() ) );
  connect( ui->m_resultTable->m_go_on, SIGNAL( pressed() ), this, SLOT( avanti() ) );
  connect( ui->m_resultTable->m_exit, SIGNAL( pressed() ), this, SLOT( esci() ) );
  connect( ui->pushButton55, SIGNAL( pressed() ), this, SLOT( primo() ) );
  connect( ui->pushButton56, SIGNAL( pressed() ), this, SLOT( secondo() ) );
  connect( ui->pushButton57, SIGNAL( pressed() ), this, SLOT( terzo() ) );
}

Form7::~Form7()
{
    delete ui;
    delete d;
}
void Form7::init()
{
    QString a[2],b[3];

 if (!d->scelta)
    d->scelta=new forlessico(d->filename);

    if (d->scelta->errore()!=-1)
    {
      azzera();
/*        d->scelta->frase(&a[0],&b[0]);
        ui->textLabel3->setText(a[0]);
        ui->textLabel4->setText(a[1]+" vuol dire:");
        ui->pushButton55->setText(b[0]);
        ui->pushButton56->setText(b[1]);
        ui->pushButton57->setText(b[2]);

        ui->m_resultTable->LCD_Wrong->display(0);
        ui->m_resultTable->LCD_Correct->display(0);
        ui->m_resultTable->m_go_on->setEnabled(false);
*/
    }
    else
    {
      QMessageBox::critical( 0, d->title, QString("Non riesco a trovare il file ") + d->filename);
      // Diamo la possibilità di ricaricare il file se non trovato
      delete d->scelta;
      d->scelta = NULL;
    }

}

void Form7::azzera()
{
    QString a[2],b[3];
    d->scelta->frase(&a[0],&b[0]);
    ui->textLabel3->setText(a[0]);
    ui->textLabel4->setText(a[1]+" vuol dire:");
    ui->pushButton55->setText(b[0]);
    ui->pushButton56->setText(b[1]);
    ui->pushButton57->setText(b[2]);
    ui->m_resultTable->LCD_Wrong->display(0);
    ui->m_resultTable->LCD_Correct->display(0);

    QPalette palette;
    palette.setColor(ui->pushButton55->backgroundRole(), QColor( 255, 255, 255 ));
    ui->pushButton55->setPalette(palette);
    palette.setColor(ui->pushButton56->backgroundRole(), QColor( 255, 255, 255 ));
    ui->pushButton56->setPalette(palette);
    palette.setColor(ui->pushButton57->backgroundRole(), QColor( 255, 255, 255 ));
    ui->pushButton57->setPalette(palette);

    ui->m_resultTable->m_go_on->setEnabled(false);
    ui->pushButton55->setEnabled(true);
    ui->pushButton56->setEnabled(true);
    ui->pushButton57->setEnabled(true);
    
}
void Form7::avanti()
{
    QString a[2],b[3];
    d->scelta->frase(&a[0],&b[0]);
    ui->textLabel3->setText(a[0]);
    ui->textLabel4->setText(a[1]+" vuol dire:");
    ui->pushButton55->setText(b[0]);
    ui->pushButton56->setText(b[1]);
    ui->pushButton57->setText(b[2]);

    QPalette palette;
    palette.setColor(ui->pushButton55->backgroundRole(), QColor( 255, 255, 255 ));
    ui->pushButton55->setPalette(palette);
    palette.setColor(ui->pushButton56->backgroundRole(), QColor( 255, 255, 255 ));
    ui->pushButton56->setPalette(palette);
    palette.setColor(ui->pushButton57->backgroundRole(), QColor( 255, 255, 255 ));
    ui->pushButton57->setPalette(palette);

    ui->m_resultTable->m_go_on->setEnabled(false);
    ui->pushButton55->setEnabled(true);
    ui->pushButton56->setEnabled(true);
    ui->pushButton57->setEnabled(true);
}

void Form7::esci()
{
    ui->m_resultTable->m_go_on->setEnabled(false);
    this->accept();
}

void Form7::primo()
{
    if (d->scelta->esatto(ui->pushButton55->text())>-1)
    {
        ui->m_resultTable->LCD_Correct->display(ui->m_resultTable->LCD_Correct->intValue()+1);
        QPalette palette;
        palette.setColor(ui->pushButton55->backgroundRole(), QColor( 255, 255, 255 ));
        ui->pushButton55->setPalette(palette);

        ui->m_resultTable->m_go_on->setEnabled(true);
        ui->pushButton55->setEnabled(false);
        ui->pushButton56->setEnabled(false);
        ui->pushButton57->setEnabled(false);
    }
    else
        ui->m_resultTable->LCD_Wrong->display(ui->m_resultTable->LCD_Wrong->intValue()+1);

}

void Form7::secondo()
{
    if (d->scelta->esatto(ui->pushButton56->text())>-1)
    {
        ui->m_resultTable->LCD_Correct->display(ui->m_resultTable->LCD_Correct->intValue()+1);
        QPalette palette;
        palette.setColor(ui->pushButton56->backgroundRole(), QColor( 255, 255, 255 ));
        ui->pushButton56->setPalette(palette);

        ui->m_resultTable->m_go_on->setEnabled(true);
        ui->pushButton55->setEnabled(false);
        ui->pushButton56->setEnabled(false);
        ui->pushButton57->setEnabled(false);
    }
    else
        ui->m_resultTable->LCD_Wrong->display(ui->m_resultTable->LCD_Wrong->intValue()+1);
}

void Form7::terzo()
{
    if (d->scelta->esatto(ui->pushButton57->text())>-1)
    {
        ui->m_resultTable->LCD_Correct->display(ui->m_resultTable->LCD_Correct->intValue()+1);
        QPalette palette;
        palette.setColor(ui->pushButton57->backgroundRole(), QColor( 255, 255, 255 ));
        ui->pushButton57->setPalette(palette);

        ui->m_resultTable->m_go_on->setEnabled(true);
        ui->pushButton55->setEnabled(false);
        ui->pushButton56->setEnabled(false);
        ui->pushButton57->setEnabled(false);
    }
    else
        ui->m_resultTable->LCD_Wrong->display(ui->m_resultTable->LCD_Wrong->intValue()+1);
}

