/***************************************************************************
 *   Copyright (C) 2006 by Nicola De Filippo   *
 *   nicola@nicoladefilippo.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "expression.h"
#include "operation.h"
#include "simbol.h"
#include <sstream>
#include <iostream>
#include <string.h>
#include <stdlib.h>

Expression::Expression(char *exp)
{
	//setExpr(exp);

	Simbol op;
	string arg_s;
	Operation conf;
	ExpElem node;
	bool isNumerator = true;
	bool isDenominator = false;
	bool error = false;
	double numerator;
	double denominator;

	text = exp;
	arg_s = "";

	if(expression.size() > 0)
		expression.clear();

	// build expression

	for(unsigned int i = 0; i < strlen(exp); i++)
	{
		if((exp[i] >= '0' && exp[i] <= '9') || exp[i] == '.' ||
		(i == 0 && exp[i] == '-'  && (exp[i + 1] >= '0' && exp[i + 1] <= '9')) || ( exp[i] == '-' && exp[i - 1] == '(' ))
			arg_s += exp[i];
		else if(exp[i] == '/')
		{
			if(arg_s.length() > 0)
			{
				numerator = atof(arg_s.c_str());
				isNumerator = false;
				isDenominator = true;
				arg_s = "";
			}	
			else
			{
				error = true;
				break;
			}
		}
		else if(conf.isSimbol(exp[i]))
		{
			// if before of the simbol was a number insert it
			if(arg_s.length() > 0)
			{
				if(isDenominator)
					denominator = atof(arg_s.c_str());
				else	
					denominator = 1;
				
				if(isNumerator)
					numerator = atof(arg_s.c_str());

				node.number.set(numerator, denominator);
				node.isnumber = true;
				expression.push_back(node);
			}
			
			// insert a simbol
			node.op.setOperation(exp[i]);
			node.op.setPriority(conf.priority(exp[i]));
			node.isnumber = false;
			expression.push_back(node);
			arg_s = "";
			isNumerator = true;
			isDenominator = false;
			arg_s = "";
			

		}
	}
	// if last element expression is a number, insert it
	if(arg_s.length() > 0)
	{
		if(isDenominator)
			denominator = atof(arg_s.c_str());
		else	
			denominator = 1;
		
		if(isNumerator)
			numerator = atof(arg_s.c_str());

		node.number.set(numerator, denominator);
		node.isnumber = true;
		expression.push_back(node);
	}
	
	
	/* 
	   If aren't errors, expression is valid and expression dont' terminate with
	   operation (+, -, *, :, /)
	*/
		
	error = error || !valid() || (!expression[expression.size() - 1].isnumber && 
		(expression[expression.size() - 1].op.sign() != '(' &&  
		expression[expression.size() - 1].op.sign() != ')'));
	
	if(error)
	{
		expression.clear();
		m_valid = false;
	}
	else
	{
		normalize();
		m_valid = true;
	}
	
	
	

}

void Expression::setExpr(const char *exp)
{
	Simbol op;
	string arg_s;
	Operation conf;
	ExpElem node;
	bool isNumerator = true;
	bool isDenominator = false;
	bool error = false;
	double numerator;
	double denominator;

	text = exp;
	arg_s = "";

	if(expression.size() > 0)
		expression.clear();

	// build expression

	for(unsigned int i = 0; i < strlen(exp); i++)
	{
		if((exp[i] >= '0' && exp[i] <= '9') || exp[i] == '.' ||
		(i == 0 && exp[i] == '-'  && (exp[i + 1] >= '0' && exp[i + 1] <= '9')) || ( exp[i] == '-' && exp[i - 1] == '(' ))
			arg_s += exp[i];
		else if(exp[i] == '/')
		{
			if(arg_s.length() > 0)
			{
				numerator = atof(arg_s.c_str());
				isNumerator = false;
				isDenominator = true;
				arg_s = "";
			}	
			else
			{
				error = true;
				break;
			}
		}
		else if(conf.isSimbol(exp[i]))
		{
			// if before of the simbol was a number insert it
			if(arg_s.length() > 0)
			{
				if(isDenominator)
					denominator = atof(arg_s.c_str());
				else	
					denominator = 1;
				
				if(isNumerator)
					numerator = atof(arg_s.c_str());

				node.number.set(numerator, denominator);
				node.isnumber = true;
				expression.push_back(node);
			}
			
			// insert a simbol
			node.op.setOperation(exp[i]);
			node.op.setPriority(conf.priority(exp[i]));
			node.isnumber = false;
			expression.push_back(node);
			arg_s = "";
			isNumerator = true;
			isDenominator = false;
			arg_s = "";
			

		}
	}
	// if last element expression is a number, insert it
	if(arg_s.length() > 0)
	{
		if(isDenominator)
			denominator = atof(arg_s.c_str());
		else	
			denominator = 1;
		
		if(isNumerator)
			numerator = atof(arg_s.c_str());

		node.number.set(numerator, denominator);
		node.isnumber = true;
		expression.push_back(node);
	}
	
	error = error || !valid() || (!expression[expression.size() - 1].isnumber && 
		(expression[expression.size() - 1].op.sign() != '(' &&  
		expression[expression.size() - 1].op.sign() != ')'));
	
	if(error)
	{
		expression.clear();
		m_valid = false;
	}
	else
	{
		normalize();
		m_valid = true;
	}
	
	
	
	
	print();
}

void Expression::print()
{

        /*for(unsigned int i = 0; i < expression.size(); i++)
	{
		if(expression[i].isnumber)
		{
			
                        cout << "I " << i << "STR " << expression[i].number.toString() << "\n";
		}
		else
                        cout << "I " << i << "STR " << expression[i].op.sign() << "\n";
        }*/

}

void Expression::getExpr(char *exp)
{
	string expText;

	expText = "";

	for(unsigned int i = 0; i < expression.size(); i++)
	{
		if(expression[i].isnumber)
		{
			
			expText += expression[i].number.toString();	
				
		}
		else
			expText += expression[i].op.sign();
	}
	text = expText;
	strcpy(exp,text.c_str());
}



void Expression::replaceNumber(string& newExpression, unsigned int pos, Z newNumber)
{
	newExpression = "";
	if(pos < expression.size() && expression[pos].isnumber)
	{
		expression[pos].number = newNumber;
		for(unsigned int i = 0; i < expression.size(); i++)
		{	
			if(expression[i].isnumber)
			{
				newExpression += expression[i].number.toString();
			}
			else
				newExpression += expression[i].op.sign();
		}
	}
		 
}


/*!
 * Viene restituita la nuova espressione in base alla poszione dell'operazione
 * @param ntext conterra' la nuova operazione
 * @param pos e' la posizione dell'operazione eseguita
 * @param res e' il risultato dell'operazione eseguita
 */


void Expression::_getExpr(string& ntext, unsigned int pos, Z &res)
{
	string expText;
	ostringstream num, den;
	expText = "";
	unsigned int upper, lower;
	bool parentesi = false;


	if(pos > 0)
	{
		// caso - a * b (dove * = qualsiasi operaziome)
		
                //cout << "IN _getExpr \n";
		
		if(pos > 2 && !expression[pos - 2].isnumber && res > 0 && expression[pos - 2].op.sign() == '-' &&
			expression[pos - 1].isnumber && expression[pos + 1].isnumber)
		{
                        //cout << "+++++++++++++++++++++ CAMBIO 1++++++++++++++++++++++++++++++\n";
                        //cout << res.toString() << "\n";
			expression[pos - 2].op.setOperation('+');
			upper = pos + 2;
			lower = pos - 1;
		}
		else if(pos > 2 && !expression[pos - 2].isnumber && res <= 0 && expression[pos - 2].op.sign() == '-' &&
		expression[pos - 1].isnumber && expression[pos + 1].isnumber) /* ||
		(!expression[pos + 1].isnumber && expression[pos - 2].op.sign() == '(') ||
		(!expression[pos + 1].isnumber && expression[pos + 1].op.sign() == '(')))*/
		{
                        //cout << "+++++++++++++++++++++ CAMBIO 2++++++++++++++++++++++++++++++\n";
			//expression[pos - 2].op.setOperation('+');
			//res = 0.0 - res;	
			upper = pos + 2;
			lower = pos - 2;
		}
		else if (pos > 2 && !expression[pos - 2].isnumber && res <= 0 && expression[pos - 2].op.sign() == '-' &&
		expression[pos - 1].isnumber && !expression[pos + 1].isnumber && expression[pos + 1].op.sign() == '(')
		{
                        //cout << "+++++++++++++++++++++ CAMBIO 3++++++++++++++++++++++++++++++\n";
			//expression[pos - 2].op.setOperation('+');
			res = 0.0 - res;	
			upper = pos + 4;
			lower = pos - 1;
		}
				
		
		/* caso *a*b (dove * = qualsiasi operazione; nota il caso '-' avanti alla 'a' e' risolto
		dall'if precedente) */
		
		else if(expression[pos - 1].isnumber && expression[pos + 1].isnumber)
		{
			
			if(pos == 1)
			{
				lower = 0;
				upper = pos + 2;
			}
			else
			{
				if(res >= 0)
				{
					lower = pos - 1;
					upper = pos + 2;
				}
				else
				{
					parentesi = true;
					lower = pos - 1;
					upper = pos + 2;
				}
			}
		}
		// caso a*(b)
		else if(!expression[pos + 1].isnumber && expression[pos + 1].op.sign() == '(')
		{
			if(pos == 1)
			{
				lower = 0;
				upper = pos + 4;
			}
			else
			{
				if(res >= 0)
				{
					// -a*(b*) >= 0
                                        if((pos - 1) >= 0 && !expression[pos - 1].isnumber && expression[pos - 1].op.sign() == '-')
					{
						expression[pos - 1].op.setOperation('+');
					}
					lower = pos - 1;
					upper = pos + 4;
				}
				else
				{
					/*if(pos - 2 >= 0 && !expression[pos - 2].isnumber && expression[pos - 2].op.sign() == '-')
					{
						expression[pos - 2].op.setOperation('+');
					}*/
					lower = pos - 1;
					parentesi = true;
					upper = pos + 4;
				}
			}
		}
		// caso (b)*a
		else if(!expression[pos - 1].isnumber && expression[pos - 1].op.sign() == ')')
		{
			if(pos == 1)
			{
				lower = 0;
				upper = pos + 2;
			}
			else
			{
				if(res >= 0)
				{
					// -(b)*a >= 0
                                        if((pos - 4) >= 0 && !expression[pos - 4].isnumber && expression[pos - 4].op.sign() == '-')
					{
						expression[pos - 4].op.setOperation('+');
                                                //cout << "CAMBIO in *++++++++++++++++++++++++++++++\n";
					}
					lower = pos - 3;
					upper = pos + 2;
				}
				else
				{
					parentesi = true;
					upper = pos + 2;
					lower = pos -3;
				}
			}
		}
		if(lower > 0)
			for(unsigned int i = 0; i < lower; i++)
			{	
				if(expression[i].isnumber)
				{
					expText += expression[i].number.toString();
				}
				else
					expText += expression[i].op.sign();
			}
		if(parentesi && expText[expText.size() - 1] != '(')
		{
			expText += '(';
			expText += res.toString();
			expText += ')';
		}
		else
			expText += res.toString();
			
		for(unsigned int i = upper; i < expression.size() ; i++)
		{	
			if(expression[i].isnumber)
			{
				expText += expression[i].number.toString();
			}
			else
				expText += expression[i].op.sign();
		}
		
	}
	ntext = expText;

#if 0
	if(pos > 0)
	{
		#if 1
		if(pos > 2 && !expression[pos - 2].isnumber && res > 0 && expression[pos - 2].op.sign() == '-')
		{
                        //cout << "+++++++++++++++++++++ CAMBIO ++++++++++++++++++++++++++++++\n";
			expression[pos - 2].op.setOperation('+');
		}
		else if(pos > 2 && !expression[pos - 2].isnumber && res < 0 && expression[pos - 2].op.sign() == '-')
		{
                        //cout << "+++++++++++++++++++++ CAMBIO ++++++++++++++++++++++++++++++\n";
			//expression[pos - 2].op.setOperation('+');
			res = 0 - res;
		}
		#endif
		for(unsigned int i = 0; i < pos - 1; i++)
		{	
			if(expression[i].isnumber)
			{
				expText += expression[i].number.toString();
			}
			else
				expText += expression[i].op.sign();
		}
	}
	
	//if((res < 0 && pos > 1) && ((!expression[pos - 2].isnumber && expression[pos - 2].op.sign() != '(' && (pos + 2) >= expression.size()) || (!expression[pos - 2].isnumber && expression[pos - 2].op.sign() != '(' && !expression[pos + 2].isnumber && expression[pos + 2].op.sign() != ')'))) /*&& expression[pos + 1].op.sign() != '('*/  
	if((res < 0 && pos > 1) && 
	((!expression[pos - 2].isnumber && expression[pos - 2].op.sign() != '('))) // && (pos + 2) >= expression.size()) || (!expression[pos - 2].isnumber && expression[pos - 2].op.sign() != '(' && !expression[pos + 2].isnumber && expression[pos + 2].op.sign() != ')')))
	{
		/*if(!expression[pos - 2].isnumber && expression[pos - 2].op.sign() == '-')
		{
			res = 0 - res;
			expText += res.toString();
		}
		else*/
			expText += "(" + res.toString() + ")";	
	}
	else	
		expText += res.toString();

	


	if((pos + 3) < expression.size() && !expression[pos + 1].isnumber && !expression[pos + 3].isnumber && expression[pos + 1].op.sign() == '(' && expression[pos + 3].op.sign() == ')')
		upper = pos + 4;
	else	
		upper = pos + 2;

		
	if(upper < expression.size())
	{
		
		for(unsigned int i = upper; i < expression.size(); i++)
		{
			if(expression[i].isnumber)
			{
				expText += expression[i].number.toString();
			}
			else
				expText += expression[i].op.sign();
		}
	}
		
	ntext = expText;
#endif
}

int Expression::getLenExpr()
{
	return text.length();
}	

int Expression::getNextOperation(int &pos)
{
	
	for(unsigned int i = 0; i < expression.size(); i++)
	{
		
		if(!verificOperation(i))
		{
			pos = i;
			return 0;
		}
		
	}
	return -1; //not valid operation
}


int Expression::getPosStr(unsigned int posElem)
{
	string dum;

	if(posElem >= expression.size())
		return -1;

	for(unsigned int i = 0; i <= posElem; i++)
	{
		if(expression[i].isnumber)
		{
			dum += expression[i].number.toString();
		}
		else
			dum += expression[i].op.sign();
	}
	
	return dum.size() - 1;
}

int Expression::exec(int pos, Z &res)
{
	bool findArg1, findArg2;
	Z arg1(0, 1);
	Z arg2;
	bool prevMinus = false;
	findArg1 = false;
	findArg2 = false;
		
	if(expression.size() == 0 || pos >= expression.size() || pos == 0)
		return -1; //error
	
	print();
	// if pos is a valid operation

	if(!expression[pos].isnumber && expression[pos].op.sign() != '(' &&  expression[pos].op.sign() != ')')
	{
		
		if(expression[pos - 1].isnumber && expression[pos + 1].isnumber)
		{
			
			if(pos == 1)
			{
				arg1 = expression[pos - 1].number;
				findArg1 = true;
			}
			if(pos == expression.size() - 2)
			{
				//arg2 = expression[pos + 1].number;
				arg2 = expression[pos + 1].number;
				findArg2 = true;
			}
			if(!findArg1)
			{
				if(!expression[pos - 2].isnumber && expression[pos - 2].op.priority() <= expression[pos].op.priority())
				{
					#if 1 //funzionava
					if(expression[pos - 2].op.sign() == '-')
					{
						arg1 -= expression[pos - 1].number;
						prevMinus = true;
					}
						
					else
					#endif
						arg1 = expression[pos - 1].number;
					findArg1 = true;
				}
				else
					return -1;
			}
			if(!findArg2)
			{
				if( !expression[pos + 2].isnumber && expression[pos + 2].op.priority() <= expression[pos].op.priority())
				{
					arg2 = expression[pos + 1].number;
					findArg2 = true;
				}
				else
					return -1;
			}
			/*if(findArg1 && findArg2)
			{
				switch(expression[pos].op.sign())
				{
					case '+':
						result = arg1 + arg2;
						break;
					case '-':
						result = arg1 - arg2;
						break;
					case '/':
						result = arg1 / arg2;
						break;
					case '*':
						result = arg1 * arg2;
						break;
					case '^':
						//result = arg1 ^ arg2;
						break;
				
				}
				return 0;
			}*/
		
		}
		
		else if((pos + 3) < expression.size() &&
		expression[pos - 1].isnumber && 
		(!expression[pos + 1].isnumber && expression[pos + 2].isnumber && expression[pos + 2].number <= 0 && expression[pos + 1].op.sign() == '(' ) && 
		(!expression[pos + 3].isnumber && expression[pos + 3].op.sign() == ')'))
		{
                        //cout << "quad_tAAAAAAAAAAAAAAAAAAAAAAAAA 2\n";
			if((pos  == 1 ) ||
			(expression[pos - 2].op.priority() <= expression[pos].op.priority()))
			{
				findArg1 = true;
				findArg2 = true;
				
				
				if(expression[pos - 2].op.sign() == '-')
				{
						arg1 -= expression[pos - 1].number;
						prevMinus = true;
				}
				else // funzionava
					arg1 = expression[pos - 1].number;
				/*if(expression[pos].op.sign() == '-' &&  expression[pos + 2].number < 0)
				{
					arg2 = 0 - expression[pos + 2].number;
                                        //cout << "CAMBIOOOOOOOOOOO\n";
				}
					
				else*/
					arg2 = expression[pos + 2].number;
			}
			else
				return -1;

		}
		
		else if((pos - 3) >= 0 &&
		expression[pos + 1].isnumber && 
		(!expression[pos - 3] .isnumber && expression[pos - 2].isnumber && expression[pos - 2].number <= 0 && expression[pos - 1 ].op.sign() == ')' ) && 
		(!expression[pos - 3].isnumber && expression[pos - 3].op.sign() == '('))
		{
			
			if(((pos + 3) == (expression.size() - 1)) ||
			(expression[pos + 2].op.priority() <= expression[pos].op.priority() && 
			((pos - 4 >= 0 && (expression[pos - 4].op.priority() <= expression[pos].op.priority()) ||
			pos - 4 < 0))))
			{
				findArg1 = true;
				findArg2 = true;
				if(pos - 4 >=0 && expression[pos - 4].op.sign() == '-')
					arg1 = 0.0 - expression[pos - 2].number;
				else
					arg1 = expression[pos - 2].number;
				arg2 = expression[pos + 1].number;
			}
			else
				return -1;
		}
		
		
		
		
		if(findArg1 && findArg2)
		{
			
			switch(expression[pos].op.sign())
			{
				
				case '+':
                                        //cout << "PRIMA della somma\n";
					res = arg1;
					res += arg2;
                                        //cout << "DOPO LA SOMMA " << res.getMaxString() << "\n";
					break;
				case '-':
					res = arg1;
					res -= arg2;
					break;
				case ':':
					res = arg1 ;
					res /= arg2;
					break;
				case '*':
					res = arg1;
					res *= arg2;
					break;
				case '^':
					//result = arg1 ^ arg2;
					break;
			
			}
			string ntext;
			#if 0
			if(prevMinus && (res > 0))
			{
				Z dumRes = 0 - res;
				_getExpr(ntext,  pos, dumRes);
			}
			else if(prevMinus && (res < 0))
			{
				Z dumRes = 0 - res;
				_getExpr(ntext,  pos, dumRes)
			}
			else
			#endif
			
                        //cout << "RISULATO " << res.toString() << "MAX " << res.getMaxString() << "\n";
			
				_getExpr(ntext,  pos, res);	
			
			
			
			
                        //cout << "NTEXT "<< ntext <<"\n";
			return 0;
		}
		return -1;
		
	}

	return -1;
}

int Expression::verificOperation(int pos)
{
	Z num;
	return exec(pos, num);

}

void Expression::normalize()
{
	string expText;
	bool done;

	expText = "";
#if 0
	for(int i = 1; i < expression.size() - 2; i++)
		if(expression[i].isnumber && (!expression[i - 1].isnumber && expression[i - 1].op.sign() == '(') && (!expression[i + 1].isnumber && expression[i + 1].op.sign() == ')'))
		{
                        //cout << "I " << i << "\n";
			expression.erase(expression.begin() + i + 1);
			expression.erase(expression.begin() + i - 1); // i + 1 after before erase is i
			break;
		}

#endif

	
	

	do
	{
		if(expression.size() == 1)
			break;

		done = false;
		for(unsigned int i = 0; i < (expression.size() - 2); i++)
		{
			if(expression[i + 1].isnumber && expression[i + 1].number > 0 && (!expression[i].isnumber && expression[i].op.sign() == '(') && (!expression[i + 2].isnumber && expression[i + 2].op.sign() == ')'))
			{
				
				expression.erase(expression.begin() + i + 2);
				expression.erase(expression.begin() + i ); 
				done = true;
				break;
			}
		}
		
		for(unsigned int j = 0; j < (expression.size() - 1) ; j++)
		{
			if(!expression[j].isnumber && expression[j].op.sign() == '-' && !expression[j + 1].isnumber && expression[j + 1].op.sign() == '-')
			{
                                //cout << "QUI\n";
				expression.erase(expression.begin() + 1 );
                                //cout << " EXPR " <<  expression.size() <<"\n";
				break;
			}
				
		}
		
	}while(done);

	for(unsigned int i = 0; i < expression.size(); i++)
	{
		if(expression[i].isnumber)
		{
			expText += expression[i].number.toString();
		}
		else
			expText += expression[i].op.sign();
	}
	text = expText;
        //cout << "NORMA " << text << "\n";
}


int Expression::size()
{
	return expression.size();
}


int Expression::getFirstNumber(Z &first)
{
	for(unsigned int i = 0; i < expression.size(); i++)
	{
		if(expression[i].isnumber)
		{
			first = expression[i].number;
			return 0;
		}
	}
	return -1;
}

int Expression::mcm(int m, int n)
{
        return ((m * n)/mcd(m ,n));
}

int Expression::mcd(int m, int n)
{
        if(m == n || n == 0)
                return m;
        else if(m > n)
                return mcd(n, m % n);
        else
                return mcd(m, n % m);
}

void Expression::getVectorExpr(vector<string> &vec)
{
	string dum;
	
	for(unsigned int i = 0; i < expression.size(); i++)
	{
		dum ="";
		
		if(expression[i].isnumber)
		{
			vec.push_back(expression[i].number.toString());
		}
		else
		{
			dum += expression[i].op.sign();
			vec.push_back(dum);
		}
			

	}
}

bool Expression::valid()
{
	bool result = false;
	unsigned int i = 0;
	int opened = 0;
	int closed = 0;
	
		
	for(unsigned int j = 0; j < expression.size(); j++)
	{
		if(!expression[j].isnumber && expression[j].op.sign() == '(')
			opened++;
		else if(!expression[j].isnumber && expression[j].op.sign() == ')')
			closed++;
	}
	
	if(expression.size() == 0 || opened != closed)
		return false;
	
	
	do //while( i < expression.size() && !valid)
	{
		result = false;
		if((expression[i].isnumber && ((i == 0 || (!expression[i - 1].isnumber && expression[i - 1].op.sign() != ')')) &&
				(i == (expression.size() - 1) || ((i < (expression.size() - 1) && !expression[i + 1].isnumber && expression[i + 1].op.sign() != '('))))))
				result = true;
		else if(((!expression[i].isnumber && expression[i].op.sign() != ')' &&
			 expression[i].op.sign() != '(') && (i > 0 && ((!expression[i - 1].isnumber && expression[i - 1].op.sign() == ')') ||
			 	expression[i - 1].isnumber ) && (i != (expression.size() - 1) && (expression[i + 1].isnumber  ||
			 	(!expression[i + 1].isnumber && expression[i + 1].op.sign() != ')') )))))
				result = true;
		else if(((!expression[i].isnumber && expression[i].op.sign() == '(') &&
				(i == 0 || 
				(!expression[i - 1].isnumber && expression[i - 1].op.sign() != ')') ||
				((i < (expression.size() - 1) &&	
				(!expression[i + 1].isnumber && expression[i + 1].op.sign() == '('))))))
				result = true;
		else if(((!expression[i].isnumber && expression[i].op.sign() == ')') &&
				(i > 0 && 
				((!expression[i - 1].isnumber && expression[i - 1].op.sign() != '(') ||
				(expression[i - 1].isnumber) || 
				(i < (expression.size() - 1) &&	
				(!expression[i + 1].isnumber && expression[i + 1].op.sign() == '('))))))
				result = true;
		i += 2;
	}while( i < expression.size() && result);
	return result;
}




/*
int main()
{
	Simbol a('+',5) ,b('-',6);
	Operation c;
	Simbol d;
	Expression esp("(10.1+20.2*50.0)-60.5");
	char es[100];
	int pp;
	int pos;
	d.setOperation(')');
	printf("Priorita' di + %d\n",c.priority('+' ));

	if(a == b)
		printf("UGUALI\n");
	
	esp.getExpr(&es[0]);
	printf("EXP = %s\n",es);

	

	if(!esp.getNextOperation(pos))
                //cout <<"Prossima %d" << pos <<"\n";
	else
                //cout << "Cazzo nessuna opera\n";

	if(!esp.exec(0,pp))
		printf("RIS %d\n",pp);
	return 0;
}

*/
