#ifndef expression_H
#define expression_H

#include <string>
#include <vector>
#include "simbol.h"
#include "operation.h"
#include "z.h"

using namespace std;

class Expression
{
private:
	typedef struct exp_elem
	{
		Z number;
		Simbol op;
		bool isnumber;
	}ExpElem;

	vector <ExpElem> expression;
	string text;
	Operation simbols;
	bool m_valid;
	

protected:
	int verificOperation(int pos);
	void normalize();
	int mcm(int m, int n);
	int mcd(int m, int n);
	
	/*!
	 * Verific is a valid expression
	 * @return true is valid, false otherwise
	 */
	
	
	bool valid();
	
public:
	Expression(char *exp);
	Expression() {;}
	~Expression() {;}
	void setExpr(const char *exp);
	void getExpr(char *exp);
	int getLenExpr();	
	int getNextOperation(int &pos);
	int exec(int pos, Z &res );
	void _getExpr(string& ntext, unsigned int pos, Z &res);
	void print();
	int size();
	/*!
	 * Give position in expression return position in the string
	 * @param posElem is position in expression
	 * @return position in text, -1 is error
	*/
	int getPosStr(unsigned int posElem);

	/*!
	 * Return first number in expression 
	 * @param first will contain first element
	 * @return 0 if founded it, -1 otherwise
	 */

	int getFirstNumber(Z &first);

	/*!
	 * Get expression elements
	 * @param vec will contain expression element 
	 */

	void getVectorExpr(vector<string> &vec);


	/*!
	 * Verific that pos element in expression is a number
	 * @param pos is position of element
	 * @return true if is a number false otherwise
	 */

	bool isNumber(int pos) { return expression[pos].isnumber; }
	
	/*!
	 * Replace number in pos with newNumber
	 * @param newExpression will contain expression with sostitution
	 * @pos is position of the number that must changed
	 * @param newNumber new value in pos
	 * @return if ok newExpression will contain new expression, string NULL otherwise
	 */
	void replaceNumber(string& newExpression, unsigned int pos, Z newNumber);

	/*!
	 * Verific that expression is a valid 
	 * @return true is valid, false otherwise
	 */
	
	bool isValid() {return m_valid;}
		
};




#endif
